#include "InformationManager.hxx"
#include "disassembler/llvm/LLVMDisassembler.hxx"
#include "core/Function.hxx"
#include "core/BasicBlock.hxx"
#include "core/Comment.hxx"

#include "gui/qt.hxx"
#include <quazip/quazip.h>
#include <quazip/quazipfile.h>

InformationManager::~InformationManager() {
	for (auto b : blocks)
		delete b.second;

	for (auto f : functions)
		delete f.second;
}

void InformationManager::reset(const std::string& filename) {
	disassembler.reset(createLLVMDisassembler(filename, this));
	if (disassembler.get() != NULL)
		disassembler.get()->start();
}

void InformationManager::save(const QString& filename) {
	QuaZip zip(filename);
	zip.open(QuaZip::mdCreate);
	zip.setComment("FRIDA 0.0");
	QuaZipFile outZipFile(&zip);

	for (auto funpair : functions) {
		Function* fun = funpair.second;
		QuaZipNewInfo zipinfo(fun->getName().c_str());
		zipinfo.setPermissions(static_cast<QFile::Permissions>(0x6444));
		outZipFile.open(QIODevice::WriteOnly, zipinfo);
		QXmlStreamWriter stream(&outZipFile);
		stream.setAutoFormatting(true);
		stream.setAutoFormattingIndent(-1);
		stream.writeStartDocument();

		fun->serialize(stream);

		stream.writeEndDocument();
		outZipFile.close();
	}

	zip.close();
}

void InformationManager::signal_new_function(Function* fun) {
}

Function* InformationManager::getFunction(uint64_t address) {
	auto it = functions.find(address);
	if (it != functions.end())
		return it->second;
	else
		return NULL;
}

BasicBlock* InformationManager::getBasicBlock(uint64_t address) {
	auto it = blocks.find(address);
	if (it != blocks.end())
		return it->second;
	else
		return NULL;
}

Function* InformationManager::newFunction(uint64_t address) {
	Function* fun = new Function(address, this);
	functions.insert(std::make_pair(address, fun));
	return fun;
}

BasicBlock* InformationManager::newBasicBlock(uint64_t address) {
	BasicBlock* block = new BasicBlock(address, this);
	blocks.insert(std::make_pair(address, block));
	return block;
}

Comment* InformationManager::newGlobalComment(uint64_t address) {
	return NULL;
}

Comment* InformationManager::newLocalComment(uint64_t address, Function* f) {
	return NULL;
}

void InformationManager::finishFunction(Function* fun) {
	for (auto b : fun->blocks()) {
		BasicBlock* bl = b.second;
		blocks.insert(std::make_pair(bl->getStartAddress(), bl));
	}
	new_function_signal(fun);
}

void InformationManager::finishBasicBlock(BasicBlock* b) {
}

void InformationManager::finnishComment(Comment* c) {
}

void InformationManager::deleteFunction(Function* f) {
	functions.erase(f->getStartAddress());
	delete f;
}

void InformationManager::deleteBasicBlock(BasicBlock* b) {
	blocks.erase(b->getStartAddress());
	delete b;
}

void InformationManager::deleteComment(Comment* c) {
	delete c;
}
